package particles;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.List;

/* Base class for executing graphics commands. This is used so that your
 * ParticleSystem type can keep track of particles in a coordinate
 * system that assumes (0, 0) is the upper-left corner, while in
 * reality the particles will be drawn centered in the screen.
 *
 * The other functions here are used by Scenes for drawing non-particle
 * objects on the screen.
 *
 * By default, none of these operations do anything. Subclasses can
 * override this behavior.
 */
public class ParticleGraphics {
    /* Set/get current drawing color. */
    public void  setColor(Color color) {}
    public Color getColor() { return Color.BLACK; }
    
    /* Draw a particle at the indicated position. */
    public void  drawParticle(double x, double y) { }
    
    /* Rectangles. */
    public void  drawRect(double x, double y, double width, double height) { }
    public void  fillRect(double x, double y, double width, double height) { }
    
    /* Ovals. */
    public void  drawOval(double x, double y, double width, double height) { }
    public void  fillOval(double x, double y, double width, double height) { }
    
    /* Polygons. */
    public void  drawPoly(List<Point2D.Double> pts) { }
    public void  fillPoly(List<Point2D.Double> pts) { }
    
    /* Lines. */
    public void  drawLine(double x0, double y0, double x1, double y1) { }
}
