package particles;
import java.util.Random;
import java.awt.Color;

/* Utility type that can be used to generate random numbers. Use this for randomness
 * generation rather than making your own Random object.
 */
public class RandomGenerator {    
    /* Produces a random color. */
    public static Color randomColor() {
        return new Color(theRandom.nextInt() & 0xFF,
                         theRandom.nextInt() & 0xFF,
                         theRandom.nextInt() & 0xFF);
    }
    
    /* Produces a random real number in the given range. */
    public static double randomReal(double low, double high) {
        return low + theRandom.nextDouble() * (high - low);
    }
    
    /* Produces a random integer in the given range. */
    public static int randomInteger(int low, int high) {
        return low + theRandom.nextInt(high - low + 1);
    }
    
    /* Returns a boolean that is true with the specified probability. */
    public static boolean randomChance(double probability) {
        if (probability < 0 || probability > 1) {
            throw new RuntimeException("Probabilities must be between 0 and 1, inclusive.");
        }
        
        return theRandom.nextDouble() < probability;
    }
    
    /* Don't want folks instantiating this. */
    private RandomGenerator() {
    
    }
    
    /* The actual random source. */
    private static final Random theRandom = new Random();
}
