package particles.scenes;
import particles.*;
import java.awt.Color;

public class Fireworks extends Scene {
    private ParticleSystem system = new ParticleSystem();
    
    public Fireworks() {
        setFramesPerSecond(20);
    }
    
    @Override public void tick() {
        /* Maybe launch another rocket! */
        if (RandomGenerator.randomChance(0.2)) {
            /* Pick a random x coordinate. */
            double x = RandomGenerator.randomReal(0, SCENE_WIDTH - 1);

            /* Launch from the bottom. */
            double y = SCENE_HEIGHT - 1;

            /* Horizontal movement is random. */
            double dx = RandomGenerator.randomReal(-5, +5);

            /* Determine a launch speed to put us toward the upper region of the window
             * (between the 1/4 and 5/6 point). Thanks, physics!
             *
             * The minus sign is here because positive y moves down, but we want this
             * rocket to move upward.
             */
            double dy = -Math.sqrt(2 * RandomGenerator.randomReal(y / 4, 5 * y / 6));

            /* Set it up as a particle. */
            Particle particle = new Particle();
            particle.x = x;
            particle.y = y;

            particle.dx = dx;
            particle.dy = dy;

            /* At each time step dy decreases by one. When it reaches zero it will be
             * hovering, and that's a great time for it to explode.
             */
            particle.lifetime = (int) -dy;

            particle.color = Color.WHITE;
            particle.type  = ParticleType.FIREWORK;

            /* Launch! */
            system.add(particle);
        }

        system.moveParticles();
    }
    
    @Override public void draw(ParticleGraphics g) {
        /* Make the whole scene area black. */
        g.setColor(Color.BLACK);
        g.fillRect(0, 0, SCENE_WIDTH, SCENE_HEIGHT);

        /* Draw the particles, including both the fireworks and the
         * streamers.
         */
        system.drawParticles(g);
    }
}
