package particles.scenes;
import particles.*;

/* Abstract base class representing a graphical scene. Graphical scenes
 * maintain a particle system and can respond to two classes of updates:
 * tick(), which advances the scene forward one step, and draw(), which
 * actually draws the scene.
 */
public abstract class Scene {
    /* For convenience, import these from ParticleSystem. */
    protected static final double SCENE_WIDTH  = ParticleSystem.SCENE_WIDTH;
    protected static final double SCENE_HEIGHT = ParticleSystem.SCENE_HEIGHT;

    public abstract void tick();
    public abstract void draw(ParticleGraphics g);
    
    /* React to the mouse, if you want. Meant to be overridden.*/
    public void mousePressed(double x, double y) {
    
    }
    
    public void mouseReleased(double x, double y) {
    
    }
    
    public void mouseDragged(double x, double y) {
    
    }
    
    public void mouseMoved(double x, double y) {
    
    }
    
    /* Frame rate defaults to 50. This can be changed, though. */
    private int fps = 50;
    
    protected void setFramesPerSecond(int fps) {
        if (fps <= 0) {
            throw new RuntimeException("FPS must be positive.");
        }
        
        this.fps = fps;
    }
    
    public int getFramesPerSecond() {
        return fps;
    }
}
