package particles.scenes;
import particles.*;
import java.awt.Color;

public class SnowyDay extends Scene {
    private ParticleSystem system = new ParticleSystem();

    /* private static finalants controlling the width and height of the window. */
    private static final double kWindowWidth  = SCENE_WIDTH  * 0.5;
    private static final double kWindowHeight = SCENE_HEIGHT * 0.8;

    /* Width of the crossbars on the window. */
    private static final double kCrossbarSize = 10;

    /* Colors. */
    private static final Color kWallColor   = new Color(127, 127, 127);
    private static final Color kBorderColor = new Color(127,  63,   0);
    private static final Color kSnowColor   = new Color(255, 255, 255);
    private static final Color kSkyColor    = new Color(  0,   0,  63);

    /* Particle parameters. */

    /* Probability of a particle being generated at each pixel across the
     * width of the scene. This is chosen so we average a target number of
     * particles per frame.
     */
    private static final double kParticleProbability = 20 / SCENE_WIDTH;

    /* Velocity parameters. */
    private static final double kDxRange = 2;
    private static final double kDownSpeed = 3;

    @Override public void tick() {
        /* Possibly add particles all across the top row. */
        for (int x = 0; x < SCENE_WIDTH; x++) {
            if (RandomGenerator.randomChance(kParticleProbability)) {
                Particle snowflake = new Particle();

                /* Originate from the top. */
                snowflake.x     = x;
                snowflake.y     = 0;

                /* Wind effects. */
                snowflake.dy    = kDownSpeed;
                snowflake.dx    = RandomGenerator.randomReal(-kDxRange, +kDxRange);

                /* Proper color. */
                snowflake.color = kSnowColor;
                system.add(snowflake);
            }
        }

        /* Now move all the particles. */
        system.moveParticles();
    }

    @Override public void draw(ParticleGraphics g) {
        /* Fill the background with the sky color. */
        g.setColor(kSkyColor);
        g.fillRect(0, 0, SCENE_WIDTH, SCENE_HEIGHT);

        /* Draw all the snowflakes. */
        system.drawParticles(g);

        /* Overdraw the walls, leaving a gap for the window. */
        g.setColor(kWallColor);

        g.fillRect(0, 0,  SCENE_WIDTH, (SCENE_HEIGHT - kWindowHeight) / 2.0); // Top
        g.fillRect(0, 0, (SCENE_WIDTH  -  kWindowWidth) / 2.0, SCENE_HEIGHT); // Left
        g.fillRect(0, (SCENE_HEIGHT + kWindowHeight) / 2.0,
                   SCENE_WIDTH, (SCENE_HEIGHT - kWindowHeight) / 2.0); // Bottom
        g.fillRect((SCENE_WIDTH  +  kWindowWidth) / 2.0, 0,
                   (SCENE_WIDTH  -  kWindowWidth) / 2.0, SCENE_HEIGHT); // Right

        /* Overdraw the crossbars. */
        g.setColor(kBorderColor);

        /* Horizontal. */
        g.fillRect((SCENE_WIDTH  -  kWindowWidth) / 2.0,
                   (SCENE_HEIGHT - kCrossbarSize) / 2.0,
                   kWindowWidth,  kCrossbarSize);

        /* Vertical. */
        g.fillRect((SCENE_WIDTH  - kCrossbarSize) / 2.0,
                   (SCENE_HEIGHT - kWindowHeight) / 2.0,
                   kCrossbarSize, kWindowHeight);

        /* Overdraw the window frame. */

        /* Left. */
        g.fillRect((SCENE_WIDTH  -  kWindowWidth) / 2.0, (SCENE_HEIGHT - kWindowHeight) / 2.0,
                   kCrossbarSize, kWindowHeight);
        /* Right. */
        g.fillRect((SCENE_WIDTH  +  kWindowWidth) / 2.0 - kCrossbarSize, (SCENE_HEIGHT - kWindowHeight) / 2.0,
                   kCrossbarSize, kWindowHeight);
        /* Top. */
        g.fillRect((SCENE_WIDTH  -  kWindowWidth) / 2.0, (SCENE_HEIGHT - kWindowHeight) / 2.0,
                   kWindowWidth, kCrossbarSize);
        /* Bottom. */
        g.fillRect((SCENE_WIDTH  -  kWindowWidth) / 2.0, (SCENE_HEIGHT + kWindowHeight) / 2.0,
                   kWindowWidth, kCrossbarSize);

    }
}
