package particles.scenes;
import particles.*;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.*;

public class Volcano extends Scene {
    private static final Color kBackgroundColor = new Color(64, 32, 32);
    private static final Color kVolcanoColor    = new Color(192, 128, 0);

    private static final double kVolcanoTopWidth = 100;

    private static final double kVolcanoHeightFraction = 0.5;

    private static final double kMaxLavaXSpeed = 10;
    private static final double kMaxSmokeXSpeed = 0.7;

    private static final int kNumLavaParticles = 6;
    private static final int kNumSmokeParticles = 15;
    
    private ParticleSystem system = new ParticleSystem();
    
    @Override public void tick() {
        /* Find where the volcano top is. */
        double volcanoTopY = (1 - kVolcanoHeightFraction) * SCENE_HEIGHT;

        /* Emit a shower of ejected lava particles. */
        for (int i = 0; i < kNumLavaParticles; i++) {
            Particle lava = new Particle();
            lava.type = ParticleType.BALLISTIC;

            double lavaOffset = RandomGenerator.randomReal(-kVolcanoTopWidth / 2, +kVolcanoTopWidth / 2);
            lava.x = SCENE_WIDTH / 2 + lavaOffset;
            lava.y = volcanoTopY;

            lava.dy = -RandomGenerator.randomReal(0, 20);
            lava.dx = lavaOffset * kMaxLavaXSpeed / (kVolcanoTopWidth / 2);

            lava.color = new Color(255, RandomGenerator.randomInteger(0, 255), 0);
            lava.lifetime = 500;

            system.add(lava);
        }

        /* Emit rising smoke. */
        for (int i = 0; i < kNumSmokeParticles; i++) {
            Particle smoke = new Particle();
            smoke.type = ParticleType.STREAMER;

            double smokeOffset = RandomGenerator.randomReal(-kVolcanoTopWidth / 2, +kVolcanoTopWidth / 2);
            smoke.x = SCENE_WIDTH / 2 + smokeOffset;
            smoke.y = volcanoTopY;

            smoke.dy = -RandomGenerator.randomReal(1, 5);
            smoke.dx = smokeOffset * kMaxSmokeXSpeed / (kVolcanoTopWidth / 2);

            int ashColor = RandomGenerator.randomInteger(128, 192);
            smoke.color = new Color(ashColor, ashColor, ashColor);
            smoke.lifetime = 500;

            system.add(smoke);
        }

        system.moveParticles();
    }
    
    @Override public void draw(ParticleGraphics graphics) {
        /* Draw background. */
        graphics.setColor(kBackgroundColor);
        graphics.fillRect(0, 0, SCENE_WIDTH, SCENE_HEIGHT);

        /* Draw the volcano. */
        double volcanoTopY = (1 - kVolcanoHeightFraction) * SCENE_HEIGHT;

        graphics.setColor(kVolcanoColor);
        
        List<Point2D.Double> volcanoPoints = new ArrayList<>();
        volcanoPoints.add(new Point2D.Double(0, SCENE_HEIGHT));
        volcanoPoints.add(new Point2D.Double(SCENE_WIDTH / 2 - kVolcanoTopWidth / 2.0, volcanoTopY));
        volcanoPoints.add(new Point2D.Double(SCENE_WIDTH / 2 + kVolcanoTopWidth / 2.0, volcanoTopY));
        volcanoPoints.add(new Point2D.Double(SCENE_WIDTH, SCENE_HEIGHT));
        graphics.fillPoly(volcanoPoints);

        /* Draw the particles, including both the fireworks and the
         * streamers.
         */
        system.drawParticles(graphics);
    }
}
